
/* Includes */

#include "hal_types.h"
#include "hal_board.h"
#include "hal_cc8051.h"

/* Defines */

#define  UART_QUEUE_RX_BUFFER_SIZE  100
#define  UART_QUEUE_TX_BUFFER_SIZE  3000

/* Variables */

// UART Queue for RX
unsigned short __xdata  uart_queue_rx_buffer[UART_QUEUE_RX_BUFFER_SIZE];
unsigned short __xdata  uart_queue_rx_point_head = 0;
unsigned short __xdata  uart_queue_rx_point_tail = 0;

// UART Queue for TX
unsigned short __xdata  uart_queue_tx_buffer[UART_QUEUE_TX_BUFFER_SIZE];
unsigned short __xdata  uart_queue_tx_point_head = 0;
unsigned short __xdata  uart_queue_tx_point_tail = 0;

/* Static Functions */

static void uart_queue_tx_increase_point_value(unsigned short * data_p)
{
    (* data_p) ++;
    if(UART_QUEUE_TX_BUFFER_SIZE == (* data_p))
    {
        (* data_p) = 0;
    }
}

static void uart_queue_rx_increase_point_value(unsigned short * data_p)
{
    (* data_p) ++;
    if(UART_QUEUE_RX_BUFFER_SIZE == (* data_p))
    {
        (* data_p) = 0;
    }
}

/* Global Functions */

void UartQueue_Initialize(void)
{
    uart_queue_rx_point_head = uart_queue_rx_point_tail = 0;
    uart_queue_tx_point_head = uart_queue_tx_point_tail = 0;
}

// UART Queue for RX

unsigned short UartQueue_Rx_Is_Empty(void)
{
    if(uart_queue_rx_point_head == uart_queue_rx_point_tail)
    {
        return TRUE;
    }
    return FALSE;
}

void UartQueue_Rx_EnQueue(unsigned short data)
{
    uart_queue_rx_buffer[uart_queue_rx_point_head] = data;
    uart_queue_rx_increase_point_value(&uart_queue_rx_point_head);
}

unsigned short UartQueue_Rx_DeQueue(void)
{
    unsigned short retVal = uart_queue_rx_buffer[uart_queue_rx_point_tail];
    uart_queue_rx_increase_point_value(&uart_queue_rx_point_tail);
    return retVal;
}

// UART Queue for TX

unsigned short UartQueue_Tx_Is_Empty(void)
{
    if(uart_queue_tx_point_head == uart_queue_tx_point_tail)
    {
        return TRUE;
    }
    return FALSE;
}

void UartQueue_Tx_EnQueue(unsigned short data)
{
    uart_queue_tx_buffer[uart_queue_tx_point_head] = data;
    uart_queue_tx_increase_point_value(&uart_queue_tx_point_head);
}

unsigned short UartQueue_Tx_DeQueue(void)
{
    unsigned short retVal = uart_queue_tx_buffer[uart_queue_tx_point_tail];
    uart_queue_tx_increase_point_value(&uart_queue_tx_point_tail);
    return retVal;
}

